searchNodes=[{"ref":"DBConnection.html","title":"DBConnection","type":"behaviour","doc":"A behaviour module for implementing efficient database connection client processes, pools and transactions. DBConnection handles callbacks differently to most behaviours. Some callbacks will be called in the calling process, with the state copied to and from the calling process. This is useful when the data for a request is large and means that a calling process can interact with a socket directly. A side effect of this is that query handling can be written in a simple blocking fashion, while the connection process itself will remain responsive to OTP messages and can enqueue and cancel queued requests. If a request or series of requests takes too long to handle in the client process a timeout will trigger and the socket can be cleanly disconnected by the connection process. If a calling process waits too long to start its request it will timeout and its request will be cancelled. This prevents requests building up when the database can not keep up. If no requests are received for an idle interval, the pool will ping all stale connections which can then ping the database to keep the connection alive. Should the connection be lost, attempts will be made to reconnect with (configurable) exponential random backoff to reconnect. All state is lost when a connection disconnects but the process is reused. The DBConnection.Query protocol provide utility functions so that queries can be encoded and decoded without blocking the connection or pool."},{"ref":"DBConnection.html#__using__/1","title":"DBConnection.__using__/1","type":"macro","doc":"Use DBConnection to set the behaviour."},{"ref":"DBConnection.html#c:checkin/1","title":"DBConnection.checkin/1","type":"callback","doc":"Checks in the state to the connection process. Return {:ok, state} to allow the checkin or {:disconnect, exception, state} to disconnect. This callback is called when the control of the state is passed back to the connection process. It should reverse any changes to the connection state made in checkout/1. This callback is called in the connection process."},{"ref":"DBConnection.html#c:checkout/1","title":"DBConnection.checkout/1","type":"callback","doc":"Checkouts the state from the connection process. Return {:ok, state} to allow the checkout or {:disconnect, exception, state} to disconnect. This callback is called when the control of the state is passed to another process. checkin/1 is called with the new state when control is returned to the connection process. This callback is called in the connection process."},{"ref":"DBConnection.html#child_spec/2","title":"DBConnection.child_spec/2","type":"function","doc":"Creates a supervisor child specification for a pool of connections. See start_link/2 for options."},{"ref":"DBConnection.html#close/3","title":"DBConnection.close/3","type":"function","doc":"Close a prepared query on a database connection and return {:ok, result} on success or {:error, exception} on error. This function should be used to free resources held by the connection process and/or the database server. Options :queue - Whether to block waiting in an internal queue for the connection&#39;s state (boolean, default: true). See &quot;Queue config&quot; in start_link/2 docs :timeout - The maximum time that the caller is allowed to perform this operation (default: 15_000) :deadline - If set, overrides :timeout option and specifies absolute monotonic time in milliseconds by which caller must perform operation. See System module documentation for more information on monotonic time (default: nil) :log - A function to log information about a call, either a 1-arity fun, {module, function, args} with DBConnection.LogEntry.t/0 prepended to args or nil. See DBConnection.LogEntry (default: nil) The pool and connection module may support other options. All options are passed to handle_close/3. See prepare/3."},{"ref":"DBConnection.html#close!/3","title":"DBConnection.close!/3","type":"function","doc":"Close a prepared query on a database connection and return the result. Raises an exception on error. See close/3."},{"ref":"DBConnection.html#c:connect/1","title":"DBConnection.connect/1","type":"callback","doc":"Connect to the database. Return {:ok, state} on success or {:error, exception} on failure. If an error is returned it will be logged and another connection attempt will be made after a backoff interval. This callback is called in the connection process."},{"ref":"DBConnection.html#c:disconnect/2","title":"DBConnection.disconnect/2","type":"callback","doc":"Disconnect from the database. Return :ok. The exception as first argument is the exception from a :disconnect 3-tuple returned by a previous callback. If the state is controlled by a client and it exits or takes too long to process a request the state will be last known state. In these cases the exception will be a DBConnection.ConnectionError. This callback is called in the connection process."},{"ref":"DBConnection.html#execute/4","title":"DBConnection.execute/4","type":"function","doc":"Execute a prepared query with a database connection and return {:ok, query, result} on success or {:error, exception} if there was an error. If the query is not prepared on the connection an attempt may be made to prepare it and then execute again. Options :queue - Whether to block waiting in an internal queue for the connection&#39;s state (boolean, default: true). See &quot;Queue config&quot; in start_link/2 docs :timeout - The maximum time that the caller is allowed to perform this operation (default: 15_000) :deadline - If set, overrides :timeout option and specifies absolute monotonic time in milliseconds by which caller must perform operation. See System module documentation for more information on monotonic time (default: nil) :log - A function to log information about a call, either a 1-arity fun, {module, function, args} with DBConnection.LogEntry.t/0 prepended to args or nil. See DBConnection.LogEntry (default: nil) The pool and connection module may support other options. All options are passed to handle_execute/4. See prepare/3."},{"ref":"DBConnection.html#execute!/4","title":"DBConnection.execute!/4","type":"function","doc":"Execute a prepared query with a database connection and return the result. Raises an exception on error. See execute/4"},{"ref":"DBConnection.html#c:handle_begin/2","title":"DBConnection.handle_begin/2","type":"callback","doc":"Handle the beginning of a transaction. Return {:ok, result, state} to continue, {status, state} to notify caller that the transaction can not begin due to the transaction status status, {:error, exception, state} (deprecated) to error without beginning the transaction, or {:disconnect, exception, state} to error and disconnect. A callback implementation should only return status if it can determine the database&#39;s transaction status without side effect. This callback is called in the client process."},{"ref":"DBConnection.html#c:handle_close/3","title":"DBConnection.handle_close/3","type":"callback","doc":"Close a query prepared by handle_prepare/3 with the database. Return {:ok, result, state} on success and to continue, {:error, exception, state} to return an error and continue, or {:disconnect, exception, state} to return an error and disconnect. This callback is called in the client process."},{"ref":"DBConnection.html#c:handle_commit/2","title":"DBConnection.handle_commit/2","type":"callback","doc":"Handle committing a transaction. Return {:ok, result, state} on successfully committing transaction, {status, state} to notify caller that the transaction can not commit due to the transaction status status, {:error, exception, state} (deprecated) to error and no longer be inside transaction, or {:disconnect, exception, state} to error and disconnect. A callback implementation should only return status if it can determine the database&#39;s transaction status without side effect. This callback is called in the client process."},{"ref":"DBConnection.html#c:handle_deallocate/4","title":"DBConnection.handle_deallocate/4","type":"callback","doc":"Deallocate a cursor declared by handle_declare/4 with the database. Return {:ok, result, state} on success and to continue, {:error, exception, state} to return an error and continue, or {:disconnect, exception, state} to return an error and disconnect. This callback is called in the client process."},{"ref":"DBConnection.html#c:handle_declare/4","title":"DBConnection.handle_declare/4","type":"callback","doc":"Declare a cursor using a query prepared by handle_prepare/3. Return {:ok, query, cursor, state} to return altered query query and cursor cursor for a stream and continue, {:error, exception, state} to return an error and continue or {:disconnect, exception, state} to return an error and disconnect. This callback is called in the client process."},{"ref":"DBConnection.html#c:handle_execute/4","title":"DBConnection.handle_execute/4","type":"callback","doc":"Execute a query prepared by handle_prepare/3. Return {:ok, query, result, state} to return altered query query and result result and continue, {:error, exception, state} to return an error and continue or {:disconnect, exception, state} to return an error and disconnect. This callback is called in the client process."},{"ref":"DBConnection.html#c:handle_fetch/4","title":"DBConnection.handle_fetch/4","type":"callback","doc":"Fetch the next result from a cursor declared by handle_declare/4. Return {:cont, result, state} to return the result result and continue using cursor, {:halt, result, state} to return the result result and close the cursor, {:error, exception, state} to return an error and close the cursor, {:disconnect, exception, state} to return an error and disconnect. This callback is called in the client process."},{"ref":"DBConnection.html#c:handle_prepare/3","title":"DBConnection.handle_prepare/3","type":"callback","doc":"Prepare a query with the database. Return {:ok, query, state} where query is a query to pass to execute/4 or close/3, {:error, exception, state} to return an error and continue or {:disconnect, exception, state} to return an error and disconnect. This callback is intended for cases where the state of a connection is needed to prepare a query and/or the query can be saved in the database to call later. This callback is called in the client process."},{"ref":"DBConnection.html#c:handle_rollback/2","title":"DBConnection.handle_rollback/2","type":"callback","doc":"Handle rolling back a transaction. Return {:ok, result, state} on successfully rolling back transaction, {status, state} to notify caller that the transaction can not rollback due to the transaction status status, {:error, exception, state} (deprecated) to error and no longer be inside transaction, or {:disconnect, exception, state} to error and disconnect. A callback implementation should only return status if it can determine the database&#39; transaction status without side effect. This callback is called in the client and connection process."},{"ref":"DBConnection.html#c:handle_status/2","title":"DBConnection.handle_status/2","type":"callback","doc":"Handle getting the transaction status. Return {:idle, state} if outside a transaction, {:transaction, state} if inside a transaction, {:error, state} if inside an aborted transaction, or {:disconnect, exception, state} to error and disconnect. If the callback returns a :disconnect tuples then status/2 will return :error."},{"ref":"DBConnection.html#c:ping/1","title":"DBConnection.ping/1","type":"callback","doc":"Called when the connection has been idle for a period of time. Return {:ok, state} to continue or {:disconnect, exception, state} to disconnect. This callback is called if no callbacks have been called after the idle timeout and a client process is not using the state. The idle timeout can be configured by the :idle_interval option. This function can be called whether the connection is checked in or checked out. This callback is called in the connection process."},{"ref":"DBConnection.html#prepare/3","title":"DBConnection.prepare/3","type":"function","doc":"Prepare a query with a database connection for later execution. It returns {:ok, query} on success or {:error, exception} if there was an error. The returned query can then be passed to execute/4 and/or close/3 Options :queue - Whether to block waiting in an internal queue for the connection&#39;s state (boolean, default: true). See &quot;Queue config&quot; in start_link/2 docs :timeout - The maximum time that the caller is allowed to perform this operation (default: 15_000) :deadline - If set, overrides :timeout option and specifies absolute monotonic time in milliseconds by which caller must perform operation. See System module documentation for more information on monotonic time (default: nil) :log - A function to log information about a call, either a 1-arity fun, {module, function, args} with DBConnection.LogEntry.t/0 prepended to args or nil. See DBConnection.LogEntry (default: nil) The pool and connection module may support other options. All options are passed to handle_prepare/3. Example DBConnection.transaction(pool, fn conn -&gt; query = %Query{statement: &quot;SELECT * FROM table&quot;} query = DBConnection.prepare!(conn, query) try do DBConnection.execute!(conn, query, []) after DBConnection.close(conn, query) end end)"},{"ref":"DBConnection.html#prepare!/3","title":"DBConnection.prepare!/3","type":"function","doc":"Prepare a query with a database connection and return the prepared query. An exception is raised on error. See prepare/3."},{"ref":"DBConnection.html#prepare_execute/4","title":"DBConnection.prepare_execute/4","type":"function","doc":"Prepare a query and execute it with a database connection and return both the prepared query and the result, {:ok, query, result} on success or {:error, exception} if there was an error. The returned query can be passed to execute/4 and close/3. Options :queue - Whether to block waiting in an internal queue for the connection&#39;s state (boolean, default: true). See &quot;Queue config&quot; in start_link/2 docs :timeout - The maximum time that the caller is allowed to perform this operation (default: 15_000) :deadline - If set, overrides :timeout option and specifies absolute monotonic time in milliseconds by which caller must perform operation. See System module documentation for more information on monotonic time (default: nil) :log - A function to log information about a call, either a 1-arity fun, {module, function, args} with DBConnection.LogEntry.t/0 prepended to args or nil. See DBConnection.LogEntry (default: nil) Example query = %Query{statement: &quot;SELECT id FROM table WHERE id=$1&quot;} {:ok, query, result} = DBConnection.prepare_execute(conn, query, [1]) {:ok, result2} = DBConnection.execute(conn, query, [2]) :ok = DBConnection.close(conn, query)"},{"ref":"DBConnection.html#prepare_execute!/4","title":"DBConnection.prepare_execute!/4","type":"function","doc":"Prepare a query and execute it with a database connection and return both the prepared query and result. An exception is raised on error. See prepare_execute/4."},{"ref":"DBConnection.html#prepare_stream/4","title":"DBConnection.prepare_stream/4","type":"function","doc":"Create a stream that will prepare a query, execute it and stream results using a cursor. Options :queue - Whether to block waiting in an internal queue for the connection&#39;s state (boolean, default: true). See &quot;Queue config&quot; in start_link/2 docs :timeout - The maximum time that the caller is allowed to perform this operation (default: 15_000) :deadline - If set, overrides :timeout option and specifies absolute monotonic time in milliseconds by which caller must perform operation. See System module documentation for more information on monotonic time (default: nil) :log - A function to log information about a call, either a 1-arity fun, {module, function, args} with DBConnection.LogEntry.t/0 prepended to args or nil. See DBConnection.LogEntry (default: nil) The pool and connection module may support other options. All options are passed to handle_prepare/3, handle_close/3, handle_declare/4, and handle_deallocate/4. Example {:ok, results} = DBConnection.transaction(conn, fn conn -&gt; query = %Query{statement: &quot;SELECT id FROM table&quot;} stream = DBConnection.prepare_stream(conn, query, []) Enum.to_list(stream) end)"},{"ref":"DBConnection.html#reduce/3","title":"DBConnection.reduce/3","type":"function","doc":"Reduces a previously built stream or prepared stream."},{"ref":"DBConnection.html#rollback/2","title":"DBConnection.rollback/2","type":"function","doc":"Rollback a database transaction and release lock on connection. When inside of a transaction/3 call does a non-local return, using a throw/1 to cause the transaction to enter a failed state and the transaction/3 call returns {:error, reason}. If transaction/3 calls are nested the connection is marked as failed until the outermost transaction call does the database rollback. Example {:error, :oops} = DBConnection.transaction(pool, fun(conn) -&gt; DBConnection.rollback(conn, :oops) end)"},{"ref":"DBConnection.html#run/3","title":"DBConnection.run/3","type":"function","doc":"Acquire a lock on a connection and run a series of requests on it. The return value of this function is the return value of fun. To use the locked connection call the request with the connection reference passed as the single argument to the fun. If the connection disconnects all future calls using that connection reference will fail. run/3 and transaction/3 can be nested multiple times but a transaction/3 call inside another transaction/3 will be treated the same as run/3. Options :queue - Whether to block waiting in an internal queue for the connection&#39;s state (boolean, default: true). See &quot;Queue config&quot; in start_link/2 docs :timeout - The maximum time that the caller is allowed to perform this operation (default: 15_000) :deadline - If set, overrides :timeout option and specifies absolute monotonic time in milliseconds by which caller must perform operation. See System module documentation for more information on monotonic time (default: nil) The pool may support other options. Example {:ok, res} = DBConnection.run(conn, fn conn -&gt; DBConnection.execute!(conn, query, []) end)"},{"ref":"DBConnection.html#start_link/2","title":"DBConnection.start_link/2","type":"function","doc":"Starts and links to a database connection process. By default the DBConnection starts a pool with a single connection. The size of the pool can be increased with :pool_size. A separate pool can be given with the :pool option. Options :backoff_min - The minimum backoff interval (default: 1_000) :backoff_max - The maximum backoff interval (default: 30_000) :backoff_type - The backoff strategy, :stop for no backoff and to stop, :exp for exponential, :rand for random and :rand_exp for random exponential (default: :rand_exp) :configure - A function to run before every connect attempt to dynamically configure the options, either a 1-arity fun, {module, function, args} with options prepended to args or nil where only returned options are passed to connect callback (default: nil) :after_connect - A function to run on connect using run/3, either a 1-arity fun, {module, function, args} with DBConnection.t/0 prepended to args or nil (default: nil) :after_connect_timeout - The maximum time allowed to perform function specified by :after_connect option (default: 15_000) :name - A name to register the started process (see the :name option in GenServer.start_link/3) :pool - Chooses the pool to be started :pool_size - Chooses the size of the pool :idle_interval - Controls the frequency we ping the database when the connection is idle :queue_target and :queue_interval - See &quot;Queue config&quot; below :max_restarts and :max_seconds - Configures the :max_restarts and :max_seconds for the connection pool supervisor (see the Supervisor docs) :show_sensitive_data_on_connection_error - By default, DBConnection hides all information during connection errors to avoid leaking credentials or other sensitive information. You can set this option if you wish to see complete errors and stacktraces during connection errors Example {:ok, conn} = DBConnection.start_link(mod, [idle_interval: 5_000]) Queue config Handling requests is done through a queue. When DBConnection is started, there are two relevant options to control the queue: :queue_target in milliseconds, defaults to 50ms :queue_interval in milliseconds, defaults to 1000ms Our goal is to wait at most :queue_target for a connection. If all connections checked out during a :queue_interval takes more than :queue_target, then we double the :queue_target. If checking out connections take longer than the new target, then we start dropping messages. For example, by default our target is 50ms. If all connections checkouts take longer than 50ms for a whole second, we double the target to 100ms and we start dropping messages if the time to checkout goes above the new limit. This allows us to better plan for overloads as we can refuse requests before they are sent to the database, which would otherwise increase the burden on the database, making the overload worse."},{"ref":"DBConnection.html#status/2","title":"DBConnection.status/2","type":"function","doc":"Return the transaction status of a connection. The callback implementation should return the transaction status according to the database, and not make assumptions based on client-side state. This function will raise a DBConnection.ConnectionError when called inside a deprecated transaction/3. Options See module documentation. The pool and connection module may support other options. All options are passed to handle_status/2. Example # outside of the transaction, the status is `:idle` DBConnection.status(conn) #=&gt; :idle DBConnection.transaction(conn, fn conn -&gt; DBConnection.status(conn) #=&gt; :transaction # run a query that will cause the transaction to rollback, e.g. # uniqueness constraint violation DBConnection.execute(conn, bad_query, []) DBConnection.status(conn) #=&gt; :error end) DBConnection.status(conn) #=&gt; :idle"},{"ref":"DBConnection.html#stream/4","title":"DBConnection.stream/4","type":"function","doc":"Create a stream that will execute a prepared query and stream results using a cursor. Options :queue - Whether to block waiting in an internal queue for the connection&#39;s state (boolean, default: true). See &quot;Queue config&quot; in start_link/2 docs :timeout - The maximum time that the caller is allowed to perform this operation (default: 15_000) :deadline - If set, overrides :timeout option and specifies absolute monotonic time in milliseconds by which caller must perform operation. See System module documentation for more information on monotonic time (default: nil) :log - A function to log information about a call, either a 1-arity fun, {module, function, args} with DBConnection.LogEntry.t/0 prepended to args or nil. See DBConnection.LogEntry (default: nil) The pool and connection module may support other options. All options are passed to handle_declare/4 and handle_deallocate/4. Example DBConnection.transaction(pool, fn conn -&gt; query = %Query{statement: &quot;SELECT id FROM table&quot;} query = DBConnection.prepare!(conn, query) try do stream = DBConnection.stream(conn, query, []) Enum.to_list(stream) after # Make sure query is closed! DBConnection.close(conn, query) end end)"},{"ref":"DBConnection.html#transaction/3","title":"DBConnection.transaction/3","type":"function","doc":"Acquire a lock on a connection and run a series of requests inside a transaction. The result of the transaction fun is return inside an :ok tuple: {:ok, result}. To use the locked connection call the request with the connection reference passed as the single argument to the fun. If the connection disconnects all future calls using that connection reference will fail. run/3 and transaction/3 can be nested multiple times. If a transaction is rolled back or a nested transaction fun raises the transaction is marked as failed. All calls except run/3, transaction/3, rollback/2, close/3 and close!/3 will raise an exception inside a failed transaction until the outer transaction call returns. All transaction/3 calls will return {:error, :rollback} if the transaction failed or connection closed and rollback/2 is not called for that transaction/3. Options :queue - Whether to block waiting in an internal queue for the connection&#39;s state (boolean, default: true). See &quot;Queue config&quot; in start_link/2 docs :timeout - The maximum time that the caller is allowed to perform this operation (default: 15_000) :deadline - If set, overrides :timeout option and specifies absolute monotonic time in milliseconds by which caller must perform operation. See System module documentation for more information on monotonic time (default: nil) :log - A function to log information about begin, commit and rollback calls made as part of the transaction, either a 1-arity fun, {module, function, args} with DBConnection.LogEntry.t/0 prepended to args or nil. See DBConnection.LogEntry (default: nil) The pool and connection module may support other options. All options are passed to handle_begin/2, handle_commit/2 and handle_rollback/2. Example {:ok, res} = DBConnection.transaction(conn, fn conn -&gt; DBConnection.execute!(conn, query, []) end)"},{"ref":"DBConnection.html#t:conn/0","title":"DBConnection.conn/0","type":"type","doc":""},{"ref":"DBConnection.html#t:cursor/0","title":"DBConnection.cursor/0","type":"type","doc":""},{"ref":"DBConnection.html#t:option/0","title":"DBConnection.option/0","type":"type","doc":""},{"ref":"DBConnection.html#t:params/0","title":"DBConnection.params/0","type":"type","doc":""},{"ref":"DBConnection.html#t:query/0","title":"DBConnection.query/0","type":"type","doc":""},{"ref":"DBConnection.html#t:result/0","title":"DBConnection.result/0","type":"type","doc":""},{"ref":"DBConnection.html#t:start_option/0","title":"DBConnection.start_option/0","type":"type","doc":""},{"ref":"DBConnection.html#t:status/0","title":"DBConnection.status/0","type":"type","doc":""},{"ref":"DBConnection.html#t:t/0","title":"DBConnection.t/0","type":"type","doc":"Run or transaction connection reference."},{"ref":"DBConnection.ConnectionError.html","title":"DBConnection.ConnectionError","type":"exception","doc":""},{"ref":"DBConnection.EncodeError.html","title":"DBConnection.EncodeError","type":"exception","doc":""},{"ref":"DBConnection.LogEntry.html","title":"DBConnection.LogEntry","type":"module","doc":"Struct containing log entry information."},{"ref":"DBConnection.LogEntry.html#t:t/0","title":"DBConnection.LogEntry.t/0","type":"type","doc":"Log entry information. :call - The DBConnection function called :query - The query used by the function :params - The params passed to the function (if any) :result - The result of the call :pool_time - The length of time awaiting a connection from the pool (if the connection was not already checked out) :connection_time - The length of time using the connection (if a connection was used) :decode_time - The length of time decoding the result (if decoded the result using DBConnection.Query.decode/3) :idle_time - The amount of time the connection was idle before use All times are in the native time units of the VM, see System.monotonic_time/0."},{"ref":"DBConnection.Ownership.html","title":"DBConnection.Ownership","type":"module","doc":"A DBConnection pool that requires explicit checkout and checkin as a mechanism to coordinate between processes. Options :ownership_mode - When mode is :manual, all connections must be explicitly checked out before by using ownership_checkout/2. Otherwise, mode is :auto and connections are checked out implicitly. {:shared, owner} mode is also supported so processes are allowed on demand. On all cases, checkins are explicit via ownership_checkin/2. Defaults to :auto. :ownership_timeout - The maximum time that a process is allowed to own a connection, default 120_000. This timeout exists mostly for sanity checking purposes and can be increased at will, since DBConnection automatically checks in connections whenever there is a mode change. :ownership_log - The Logger.level to log ownership changes, or nil not to log, default nil. There are also two experimental options, :post_checkout and :pre_checkin which allows a developer to configure what happens when a connection is checked out and checked in. Those options are meant to be used during tests, and have the following behaviour: :post_checkout - it must be an anonymous function that receives the connection module, the connection state and it must return either {:ok, connection_module, connection_state} or {:disconnect, err, connection_module, connection_state}. This allows the developer to change the connection module on post checkout. However, in case of disconnects, the return connection_module must be the same as the connection_module given. Defaults to simply returning the given connection module and state. :pre_checkin - it must be an anonymous function that receives the checkin reason (:checkin, {:disconnect, err} or {:stop, err}), the connection module and the connection state returned by post_checkout. It must return either {:ok, connection_module, connection_state} or {:disconnect, err, connection_module, connection_state} where the connection module is the module given to :post_checkout Defaults to simply returning the given connection module and state. Callers lookup When checking out, the ownership pool first looks if there is a connection assigned to the current process and then checks if there is a connection assigned to any of the processes listed under the $callers process dictionary entry. The $callers entry is set by default for tasks from Elixir v1.8. You can also pass the :caller option on checkout with a pid and that pid will be looked up first, instead of self(), and then we fall back to $callers."},{"ref":"DBConnection.Ownership.html#ownership_allow/4","title":"DBConnection.Ownership.ownership_allow/4","type":"function","doc":"Allows the process given by allow to use the connection checked out by owner_or_allowed. It may return :ok if the connection is checked out. {:already, :owner | :allowed} if the allow process already has a connection. owner_or_allowed may either be the owner or any other allowed process. Returns :not_found if the given process does not have any connection checked out."},{"ref":"DBConnection.Ownership.html#ownership_checkin/2","title":"DBConnection.Ownership.ownership_checkin/2","type":"function","doc":"Checks a connection back in. A connection can only be checked back in by its owner."},{"ref":"DBConnection.Ownership.html#ownership_checkout/2","title":"DBConnection.Ownership.ownership_checkout/2","type":"function","doc":"Explicitly checks a connection out from the ownership manager. It may return :ok if the connection is checked out. {:already, :owner | :allowed} if the caller process already has a connection, :error if it could be not checked out or raise if there was an error."},{"ref":"DBConnection.Ownership.html#ownership_mode/3","title":"DBConnection.Ownership.ownership_mode/3","type":"function","doc":"Changes the ownership mode. mode may be :auto, :manual or {:shared, owner}. The operation will always succeed when setting the mode to :auto or :manual. It may fail with reason :not_owner or :not_found when setting {:shared, pid} and the given pid does not own any connection. May return :already_shared if another process set the ownership mode to {:shared, _} and is still alive."},{"ref":"DBConnection.OwnershipError.html","title":"DBConnection.OwnershipError","type":"exception","doc":""},{"ref":"DBConnection.PrepareStream.html","title":"DBConnection.PrepareStream","type":"module","doc":""},{"ref":"DBConnection.PrepareStream.html#t:t/0","title":"DBConnection.PrepareStream.t/0","type":"type","doc":""},{"ref":"DBConnection.Query.html","title":"DBConnection.Query","type":"protocol","doc":"The DBConnection.Query protocol is responsible for preparing and encoding queries. All DBConnection.Query functions are executed in the caller process which means it&#39;s safe to, for example, raise exceptions or do blocking calls as they won&#39;t affect the connection process."},{"ref":"DBConnection.Query.html#decode/3","title":"DBConnection.Query.decode/3","type":"function","doc":"Decode a result using a query. This function is called to decode a result after it is returned by a connection callback module. See DBConnection.execute/3."},{"ref":"DBConnection.Query.html#describe/2","title":"DBConnection.Query.describe/2","type":"function","doc":"Describe a query. This function is called to describe a query after it is prepared using a connection callback module. See DBConnection.prepare/3."},{"ref":"DBConnection.Query.html#encode/3","title":"DBConnection.Query.encode/3","type":"function","doc":"Encode parameters using a query. This function is called to encode a query before it is executed using a connection callback module. If this function raises DBConnection.EncodeError, then the query is prepared once again. See DBConnection.execute/3."},{"ref":"DBConnection.Query.html#parse/2","title":"DBConnection.Query.parse/2","type":"function","doc":"Parse a query. This function is called to parse a query term before it is prepared using a connection callback module. See DBConnection.prepare/3."},{"ref":"DBConnection.Query.html#t:t/0","title":"DBConnection.Query.t/0","type":"type","doc":""},{"ref":"DBConnection.Stream.html","title":"DBConnection.Stream","type":"module","doc":""},{"ref":"DBConnection.Stream.html#t:t/0","title":"DBConnection.Stream.t/0","type":"type","doc":""},{"ref":"DBConnection.TransactionError.html","title":"DBConnection.TransactionError","type":"exception","doc":""}]